@@ -13,6 +13,7 @@ from .pagination import PaginatorResource, PaginatorParametizer
 
 
 class PreviewParametizer(PaginatorParametizer):
+    processor = StrParam('default', choices=pipeline.query_processors)
     tree = StrParam(MODELTREE_DEFAULT_ALIAS, choices=trees)
 
 
@@ -38,7 +39,7 @@ class PreviewResource(BaseResource, PaginatorResource):
         context = self.get_context(request)
 
         # Initialize a query processor
-        QueryProcessor = pipeline.query_processors.default
+        QueryProcessor = pipeline.query_processors[params['processor']]
         processor = QueryProcessor(context=context, view=view, tree=tree)
 
         # Build a queryset for pagination and other downstream use
