@@ -108,3 +108,12 @@ class TokenGenerator(object):
 
 
 token_generator = TokenGenerator()
+
+
+def get_request_token(request):
+    "Attempts to retrieve a token from the request."
+    if 'token' in request.REQUEST:
+        return request.REQUEST['token']
+    if 'HTTP_API_TOKEN' in request.META:
+        return request.META['HTTP_API_TOKEN']
+    return ''
